/**
 * vim: set ts=4 :
 * =============================================================================
 * SourceMod Sample Extension
 * Copyright (C) 2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "extension.h"
#include "smsdk_ext.h"

SH_DECL_MANUALHOOK0(CanBeAutobalanced, 0, 0, 0, bool);

IForward *g_OnCanBeAutobalanced;

/**
 * @file extension.cpp
 * @brief Implement extension code here.
 */

Sample g_Sample;		/**< Global singleton for extension's main interface */

SMEXT_LINK(&g_Sample);

bool Sample::SDK_OnLoad(char *error, size_t maxlength, bool late)
{
	IGameConfig *conf;
	if (!gameconfs->LoadGameConfigFile("tf2autobalance", &conf, error, maxlength))
	{
		return false;
	}

	
	int offset;
	if (!conf->GetOffset("CanBeAutobalanced", &offset))
	{
		g_pSM->Format(error, maxlength, "Could not find CanBeAutobalanced offset in gamedata/tf2autobalance.txt");
		return false;
	}

	SH_MANUALHOOK_RECONFIGURE(CanBeAutobalanced, offset, 0, 0);

	if (late)
	{
		int maxClients = playerhelpers->GetMaxClients();
		for (int i = 0; i <= maxClients; i++)
		{
			IGamePlayer *pPlayer = playerhelpers->GetGamePlayer(i);
			if (pPlayer && pPlayer->IsInGame())
			{
				SH_ADD_MANUALHOOK_MEMFUNC(CanBeAutobalanced, gamehelpers->ReferenceToEntity(i), &g_Sample, &Sample::Hook_CanBeAutobalanced, false);
			}
		}
	}

	playerhelpers->AddClientListener(this);

	g_OnCanBeAutobalanced = forwards->CreateForward("OnCanBeAutobalanced", ET_Event, 2, NULL, Param_Cell, Param_CellByRef);

	return true;
}

void Sample::SDK_OnUnload()
{
	playerhelpers->RemoveClientListener(this);
}

void Sample::OnClientPutInServer(int client)
{
	SH_ADD_MANUALHOOK_MEMFUNC(CanBeAutobalanced, gamehelpers->ReferenceToEntity(client), &g_Sample, &Sample::Hook_CanBeAutobalanced, false);
}

void Sample::OnClientDisconnecting(int client)
{
	SH_REMOVE_MANUALHOOK_MEMFUNC(CanBeAutobalanced, gamehelpers->ReferenceToEntity(client), &g_Sample, &Sample::Hook_CanBeAutobalanced, false);
}

bool Sample::Hook_CanBeAutobalanced()
{
	CBaseEntity *pPlayer = META_IFACEPTR(CBaseEntity);
	
	cell_t res;
	cell_t newret = 1;
	g_OnCanBeAutobalanced->PushCell(gamehelpers->EntityToBCompatRef(pPlayer));
	g_OnCanBeAutobalanced->PushCellByRef(&newret);
	g_OnCanBeAutobalanced->Execute(&res);

	if (res == Pl_Continue)
		RETURN_META_VALUE(MRES_IGNORED, true);

	return (newret == 0) ? false : true;
}
